@@ -44,16 +44,53 @@ namespace glz
 
 namespace glz
 {
-   // Generic json type.
-   struct generic
-   {
-      virtual ~generic() {}
+   // Number storage mode for generic JSON types
+   enum class num_mode {
+      f64, // double only - fast, JavaScript-compatible (default)
+      i64, // int64_t → double - signed integer precision
+      u64 // uint64_t → int64_t → double - full integer range
+   };
 
-      using array_t = std::vector<generic>;
-      using object_t = std::map<std::string, generic, std::less<>>;
+   // Helper to determine variant type based on mode
+   template <num_mode Mode, class null_t, class array_t, class object_t>
+   struct generic_val_type;
+
+   template <class null_t, class array_t, class object_t>
+   struct generic_val_type<num_mode::f64, null_t, array_t, object_t>
+   {
+      using type = std::variant<null_t, double, std::string, bool, array_t, object_t>;
+   };
+
+   template <class null_t, class array_t, class object_t>
+   struct generic_val_type<num_mode::i64, null_t, array_t, object_t>
+   {
+      using type = std::variant<null_t, int64_t, double, std::string, bool, array_t, object_t>;
+   };
+
+   template <class null_t, class array_t, class object_t>
+   struct generic_val_type<num_mode::u64, null_t, array_t, object_t>
+   {
+      using type = std::variant<null_t, uint64_t, int64_t, double, std::string, bool, array_t, object_t>;
+   };
+
+   // Generic json type.
+   // Template parameter controls number storage mode for precise integer handling
+   template <num_mode Mode = num_mode::f64>
+   struct generic_json
+   {
+      virtual ~generic_json() {}
+
+      using array_t = std::vector<generic_json<Mode>>;
+      using object_t = std::map<std::string, generic_json<Mode>, std::less<>>;
       using null_t = std::nullptr_t;
-      using val_t = std::variant<null_t, double, std::string, bool, array_t, object_t>;
-      val_t data{};
+
+      // Variant type depends on Mode:
+      // - f64: double only (fast, JS-compatible)
+      // - i64: int64_t, then double (signed integer precision)
+      // - u64: uint64_t, then int64_t, then double (full integer range)
+      using val_t = typename generic_val_type<Mode, null_t, array_t, object_t>::type;
+
+      val_t data;
 
       /**
        * @brief Converts the JSON data to a string representation.
@@ -102,6 +139,19 @@ namespace glz
       [[nodiscard]] T as() const
       {
          // Can be used for int and the like
+         if constexpr (Mode == num_mode::u64) {
+            if (holds<uint64_t>()) {
+               return static_cast<T>(get<uint64_t>());
+            }
+            if (holds<int64_t>()) {
+               return static_cast<T>(get<int64_t>());
+            }
+         }
+         else if constexpr (Mode == num_mode::i64) {
+            if (holds<int64_t>()) {
+               return static_cast<T>(get<int64_t>());
+            }
+         }
          return static_cast<T>(get<double>());
       }
 
@@ -119,23 +169,23 @@ namespace glz
          return std::holds_alternative<T>(data);
       }
 
-      generic& operator[](std::integral auto&& index) { return std::get<array_t>(data)[index]; }
+      generic_json& operator[](std::integral auto&& index) { return std::get<array_t>(data)[index]; }
 
-      const generic& operator[](std::integral auto&& index) const { return std::get<array_t>(data)[index]; }
+      const generic_json& operator[](std::integral auto&& index) const { return std::get<array_t>(data)[index]; }
 
-      generic& operator[](std::convertible_to<std::string_view> auto&& key)
+      generic_json& operator[](std::convertible_to<std::string_view> auto&& key)
       {
          //[] operator for maps does not support heterogeneous lookups yet
          if (holds<null_t>()) data = object_t{};
          auto& object = std::get<object_t>(data);
          auto iter = object.find(key);
          if (iter == object.end()) {
-            iter = object.insert(std::make_pair(std::string(key), generic{})).first;
+            iter = object.insert(std::make_pair(std::string(key), generic_json{})).first;
          }
          return iter->second;
       }
 
-      const generic& operator[](std::convertible_to<std::string_view> auto&& key) const
+      const generic_json& operator[](std::convertible_to<std::string_view> auto&& key) const
       {
          //[] operator for maps does not support heterogeneous lookups yet
          auto& object = std::get<object_t>(data);
@@ -146,74 +196,97 @@ namespace glz
          return iter->second;
       }
 
-      generic& operator=(const std::nullptr_t value)
+      generic_json& operator=(const std::nullptr_t value)
       {
          data = value;
          return *this;
       }
 
-      generic& operator=(const double value)
+      generic_json& operator=(const double value)
       {
          data = value;
          return *this;
       }
 
-      // for integers
-      template <int_t T>
-      generic& operator=(const T value)
+      // for unsigned integers
+      template <class T>
+         requires(std::unsigned_integral<T> && !std::same_as<T, bool>)
+      generic_json& operator=(const T value)
       {
-         data = static_cast<double>(value);
+         if constexpr (Mode == num_mode::u64) {
+            data = static_cast<uint64_t>(value);
+         }
+         else if constexpr (Mode == num_mode::i64) {
+            data = static_cast<int64_t>(value);
+         }
+         else {
+            data = static_cast<double>(value);
+         }
          return *this;
       }
 
-      generic& operator=(const std::string& value)
+      // for signed integers
+      template <class T>
+         requires(std::signed_integral<T>)
+      generic_json& operator=(const T value)
+      {
+         if constexpr (Mode == num_mode::u64 || Mode == num_mode::i64) {
+            data = static_cast<int64_t>(value);
+         }
+         else {
+            data = static_cast<double>(value);
+         }
+         return *this;
+      }
+
+      generic_json& operator=(const std::string& value)
       {
          data = value;
          return *this;
       }
 
-      generic& operator=(const std::string_view value)
+      generic_json& operator=(const std::string_view value)
       {
          data = std::string(value);
          return *this;
       }
 
-      generic& operator=(const char* value)
+      generic_json& operator=(const char* value)
       {
          data = std::string(value);
          return *this;
       }
 
-      generic& operator=(const bool value)
+      generic_json& operator=(const bool value)
       {
          data = value;
          return *this;
       }
 
-      generic& operator=(const array_t& value)
+      generic_json& operator=(const array_t& value)
       {
          data = value;
          return *this;
       }
 
-      generic& operator=(const object_t& value)
+      generic_json& operator=(const object_t& value)
       {
          data = value;
          return *this;
       }
 
       template <class T>
-         requires(!std::is_same_v<std::decay_t<T>, generic> && !std::is_same_v<std::decay_t<T>, std::nullptr_t> &&
+         requires(!std::is_same_v<std::decay_t<T>, generic_json> && !std::is_same_v<std::decay_t<T>, std::nullptr_t> &&
                   !std::is_same_v<std::decay_t<T>, double> && !std::is_same_v<std::decay_t<T>, bool> &&
                   !std::is_same_v<std::decay_t<T>, std::string> && !std::is_same_v<std::decay_t<T>, std::string_view> &&
                   !std::is_same_v<std::decay_t<T>, const char*> && !std::is_same_v<std::decay_t<T>, array_t> &&
-                  !std::is_same_v<std::decay_t<T>, object_t> && !int_t<T> &&
-                  requires { write_json(std::declval<T>()); })
-      generic& operator=(T&& value)
+                  !std::is_same_v<std::decay_t<T>, object_t> && !std::integral<std::decay_t<T>> &&
+                  !std::floating_point<std::decay_t<T>> && requires { write_json(std::declval<T>()); })
+      generic_json& operator=(T&& value)
       {
          auto json_str = write_json(std::forward<T>(value));
          if (json_str) {
-            auto result = read_json<generic>(*json_str);
+            auto result = read_json<generic_json>(*json_str);
             if (result) {
                *this = std::move(*result);
             }
@@ -221,9 +294,9 @@ namespace glz
          return *this;
       }
 
-      [[nodiscard]] generic& at(std::convertible_to<std::string_view> auto&& key) { return operator[](key); }
+      [[nodiscard]] generic_json& at(std::convertible_to<std::string_view> auto&& key) { return operator[](key); }
 
-      [[nodiscard]] const generic& at(std::convertible_to<std::string_view> auto&& key) const
+      [[nodiscard]] const generic_json& at(std::convertible_to<std::string_view> auto&& key) const
       {
          return operator[](key);
       }
@@ -246,28 +319,51 @@ namespace glz
 
       void reset() noexcept { data = null_t{}; }
 
-      generic() = default;
-      generic(const generic& other) : data(other.data) {}
-      generic& operator=(const generic&) = default;
-      generic(generic&& other) noexcept : data(std::move(other.data)) {}
-      generic& operator=(generic&&) = default;
+      generic_json() = default;
+      generic_json(const generic_json& other) : data(other.data) {}
+      generic_json& operator=(const generic_json&) = default;
+      generic_json(generic_json&& other) noexcept : data(std::move(other.data)) {}
+      generic_json& operator=(generic_json&&) = default;
 
       template <class T>
-         requires std::convertible_to<T, val_t> && (!std::derived_from<std::decay_t<T>, generic>)
-      generic(T&& val) : data(std::forward<T>(val))
+         requires std::convertible_to<T, val_t> && (!std::derived_from<std::decay_t<T>, generic_json>)
+      generic_json(T&& val) : data(std::forward<T>(val))
       {}
 
       template <class T>
-         requires std::convertible_to<T, double> && (!std::derived_from<std::decay_t<T>, generic>) &&
+         requires std::convertible_to<T, double> && (!std::derived_from<std::decay_t<T>, generic_json>) &&
                   (!std::convertible_to<T, val_t>)
-      generic(T&& val) : data(static_cast<double>(std::forward<T>(val)))
-      {}
-
-      generic(const std::string_view value) : data(std::string(value)) {}
-
-      generic(std::initializer_list<std::pair<const char*, generic>>&& obj)
+      generic_json(T&& val)
       {
-         data.emplace<object_t>();
+         if constexpr (Mode == num_mode::u64) {
+            if constexpr (std::unsigned_integral<T>) {
+               data = static_cast<uint64_t>(val);
+            }
+            else if constexpr (std::signed_integral<T>) {
+               data = static_cast<int64_t>(val);
+            }
+            else {
+               data = static_cast<double>(val);
+            }
+         }
+         else if constexpr (Mode == num_mode::i64) {
+            if constexpr (std::integral<T>) {
+               data = static_cast<int64_t>(val);
+            }
+            else {
+               data = static_cast<double>(val);
+            }
+         }
+         else {
+            data = static_cast<double>(val);
+         }
+      }
+
+      generic_json(const std::string_view value) : data(std::string(value)) {}
+
+      generic_json(std::initializer_list<std::pair<const char*, generic_json>>&& obj)
+      {
+         data.template emplace<object_t>();
          auto& data_obj = std::get<object_t>(data);
          for (auto&& pair : obj) {
             // std::initializer_list holds const values that cannot be moved
@@ -277,16 +373,27 @@ namespace glz
 
       // Prevent conflict with object initializer list
       template <bool deprioritize = true>
-      generic(std::initializer_list<generic>&& arr)
+      generic_json(std::initializer_list<generic_json>&& arr)
       {
-         data.emplace<array_t>(std::move(arr));
+         data.template emplace<array_t>(std::move(arr));
       }
 
-      [[nodiscard]] bool is_array() const noexcept { return holds<generic::array_t>(); }
+      [[nodiscard]] bool is_array() const noexcept { return holds<array_t>(); }
 
-      [[nodiscard]] bool is_object() const noexcept { return holds<generic::object_t>(); }
+      [[nodiscard]] bool is_object() const noexcept { return holds<object_t>(); }
 
-      [[nodiscard]] bool is_number() const noexcept { return holds<double>(); }
+      [[nodiscard]] bool is_number() const noexcept
+      {
+         if constexpr (Mode == num_mode::u64) {
+            return holds<uint64_t>() || holds<int64_t>() || holds<double>();
+         }
+         else if constexpr (Mode == num_mode::i64) {
+            return holds<int64_t>() || holds<double>();
+         }
+         else {
+            return holds<double>();
+         }
+      }
 
       [[nodiscard]] bool is_string() const noexcept { return holds<std::string>(); }
 
@@ -300,8 +407,84 @@ namespace glz
       [[nodiscard]] object_t& get_object() { return get<object_t>(); }
       [[nodiscard]] const object_t& get_object() const { return get<object_t>(); }
 
-      [[nodiscard]] double& get_number() { return get<double>(); }
-      [[nodiscard]] const double& get_number() const { return get<double>(); }
+      [[nodiscard]] double& get_number()
+      {
+         if constexpr (Mode == num_mode::u64) {
+            if (holds<uint64_t>()) {
+               glaze_error(
+                  "Cannot get reference to double when variant holds uint64_t. Use as<double>() for conversion.");
+            }
+            if (holds<int64_t>()) {
+               glaze_error(
+                  "Cannot get reference to double when variant holds int64_t. Use as<double>() for conversion.");
+            }
+         }
+         else if constexpr (Mode == num_mode::i64) {
+            if (holds<int64_t>()) {
+               glaze_error(
+                  "Cannot get reference to double when variant holds int64_t. Use as<double>() for conversion.");
+            }
+         }
+         return get<double>();
+      }
+      [[nodiscard]] const double& get_number() const
+      {
+         if constexpr (Mode == num_mode::u64) {
+            if (holds<uint64_t>()) {
+               glaze_error(
+                  "Cannot get reference to double when variant holds uint64_t. Use as<double>() for conversion.");
+            }
+            if (holds<int64_t>()) {
+               glaze_error(
+                  "Cannot get reference to double when variant holds int64_t. Use as<double>() for conversion.");
+            }
+         }
+         else if constexpr (Mode == num_mode::i64) {
+            if (holds<int64_t>()) {
+               glaze_error(
+                  "Cannot get reference to double when variant holds int64_t. Use as<double>() for conversion.");
+            }
+         }
+         return get<double>();
+      }
+
+      // Get number as double, converting from integer types if needed (only available in i64/u64 modes)
+      [[nodiscard]] double as_number() const
+         requires(Mode != num_mode::f64)
+      {
+         if constexpr (Mode == num_mode::u64) {
+            if (holds<uint64_t>()) {
+               return static_cast<double>(get<uint64_t>());
+            }
+         }
+         if constexpr (Mode == num_mode::u64 || Mode == num_mode::i64) {
+            if (holds<int64_t>()) {
+               return static_cast<double>(get<int64_t>());
+            }
+         }
+         return get<double>();
+      }
+
+      // Check if the number is stored as uint64_t (only available in u64 mode)
+      [[nodiscard]] bool is_uint64() const noexcept
+         requires(Mode == num_mode::u64)
+      {
+         return holds<uint64_t>();
+      }
+
+      // Check if the number is stored as int64_t (only available in i64/u64 modes)
+      [[nodiscard]] bool is_int64() const noexcept
+         requires(Mode != num_mode::f64)
+      {
+         return holds<int64_t>();
+      }
+
+      // Check if the number is stored as double (only available in i64/u64 modes)
+      [[nodiscard]] bool is_double() const noexcept
+         requires(Mode != num_mode::f64)
+      {
+         return holds<double>();
+      }
 
       [[nodiscard]] std::string& get_string() { return get<std::string>(); }
       [[nodiscard]] const std::string& get_string() const { return get<std::string>(); }
@@ -348,27 +531,56 @@ namespace glz
       }
    };
 
+   // Type aliases for different number handling modes
+   using generic = generic_json<num_mode::f64>; // double only - fast, JavaScript-compatible
+   using generic_i64 = generic_json<num_mode::i64>; // int64_t → double - signed integer precision
+   using generic_u64 = generic_json<num_mode::u64>; // uint64_t → int64_t → double - full integer range
+
    // Backwards compatibility alias
    using json_t [[deprecated("glz::json_t is deprecated, use glz::generic instead")]] = generic;
 
-   [[nodiscard]] inline bool is_array(const generic& value) noexcept { return value.is_array(); }
+   template <num_mode Mode>
+   [[nodiscard]] inline bool is_array(const generic_json<Mode>& value) noexcept
+   {
+      return value.is_array();
+   }
 
-   [[nodiscard]] inline bool is_object(const generic& value) noexcept { return value.is_object(); }
+   template <num_mode Mode>
+   [[nodiscard]] inline bool is_object(const generic_json<Mode>& value) noexcept
+   {
+      return value.is_object();
+   }
 
-   [[nodiscard]] inline bool is_number(const generic& value) noexcept { return value.is_number(); }
+   template <num_mode Mode>
+   [[nodiscard]] inline bool is_number(const generic_json<Mode>& value) noexcept
+   {
+      return value.is_number();
+   }
 
-   [[nodiscard]] inline bool is_string(const generic& value) noexcept { return value.is_string(); }
+   template <num_mode Mode>
+   [[nodiscard]] inline bool is_string(const generic_json<Mode>& value) noexcept
+   {
+      return value.is_string();
+   }
 
-   [[nodiscard]] inline bool is_boolean(const generic& value) noexcept { return value.is_boolean(); }
+   template <num_mode Mode>
+   [[nodiscard]] inline bool is_boolean(const generic_json<Mode>& value) noexcept
+   {
+      return value.is_boolean();
+   }
 
-   [[nodiscard]] inline bool is_null(const generic& value) noexcept { return value.is_null(); }
+   template <num_mode Mode>
+   [[nodiscard]] inline bool is_null(const generic_json<Mode>& value) noexcept
+   {
+      return value.is_null();
+   }
 }
 
-template <>
-struct glz::meta<glz::generic>
+template <glz::num_mode Mode>
+struct glz::meta<glz::generic_json<Mode>>
 {
    static constexpr std::string_view name = "glz::generic";
-   using T = glz::generic;
+   using T = glz::generic_json<Mode>;
    static constexpr auto value = &T::data;
 };
 
@@ -629,58 +841,150 @@ namespace glz
    // Takes result by reference to allow memory reuse and avoid extra moves
 
    // Forward declaration for recursive conversion
-   template <class T>
-   error_ctx convert_from_generic(T& result, const generic& source);
+   template <class T, num_mode Mode>
+   error_ctx convert_from_generic(T& result, const generic_json<Mode>& source);
 
    // Specialization for bool
-   template <>
-   inline error_ctx convert_from_generic<bool>(bool& result, const generic& source)
+   template <num_mode Mode>
+   inline error_ctx convert_from_generic(bool& result, const generic_json<Mode>& source)
    {
       if (!source.is_boolean()) {
          return error_ctx{error_code::syntax_error};
       }
-      result = source.get<bool>();
+      result = source.template get<bool>();
       return {};
    }
 
    // Specialization for double
-   template <>
-   inline error_ctx convert_from_generic<double>(double& result, const generic& source)
+   template <num_mode Mode>
+   inline error_ctx convert_from_generic(double& result, const generic_json<Mode>& source)
    {
       if (!source.is_number()) {
          return error_ctx{error_code::syntax_error};
       }
-      result = source.get<double>();
+      if constexpr (Mode == num_mode::u64) {
+         if (source.template holds<uint64_t>()) {
+            result = static_cast<double>(source.template get<uint64_t>());
+         }
+         else if (source.template holds<int64_t>()) {
+            result = static_cast<double>(source.template get<int64_t>());
+         }
+         else {
+            result = source.template get<double>();
+         }
+      }
+      else if constexpr (Mode == num_mode::i64) {
+         if (source.template holds<int64_t>()) {
+            result = static_cast<double>(source.template get<int64_t>());
+         }
+         else {
+            result = source.template get<double>();
+         }
+      }
+      else {
+         result = source.template get<double>();
+      }
+      return {};
+   }
+
+   // Specialization for uint64_t (only in u64 mode)
+   template <num_mode Mode>
+      requires(Mode == num_mode::u64)
+   inline error_ctx convert_from_generic(uint64_t& result, const generic_json<Mode>& source)
+   {
+      if (!source.is_number()) {
+         return error_ctx{error_code::syntax_error};
+      }
+      if (source.template holds<uint64_t>()) {
+         result = source.template get<uint64_t>();
+      }
+      else if (source.template holds<int64_t>()) {
+         result = static_cast<uint64_t>(source.template get<int64_t>());
+      }
+      else {
+         result = static_cast<uint64_t>(source.template get<double>());
+      }
+      return {};
+   }
+
+   // Specialization for int64_t (in i64 and u64 modes)
+   template <num_mode Mode>
+      requires(Mode != num_mode::f64)
+   inline error_ctx convert_from_generic(int64_t& result, const generic_json<Mode>& source)
+   {
+      if (!source.is_number()) {
+         return error_ctx{error_code::syntax_error};
+      }
+      if constexpr (Mode == num_mode::u64) {
+         if (source.template holds<uint64_t>()) {
+            result = static_cast<int64_t>(source.template get<uint64_t>());
+         }
+         else if (source.template holds<int64_t>()) {
+            result = source.template get<int64_t>();
+         }
+         else {
+            result = static_cast<int64_t>(source.template get<double>());
+         }
+      }
+      else {
+         if (source.template holds<int64_t>()) {
+            result = source.template get<int64_t>();
+         }
+         else {
+            result = static_cast<int64_t>(source.template get<double>());
+         }
+      }
       return {};
    }
 
    // Specialization for string
-   template <>
-   inline error_ctx convert_from_generic<std::string>(std::string& result, const generic& source)
+   template <num_mode Mode>
+   inline error_ctx convert_from_generic(std::string& result, const generic_json<Mode>& source)
    {
       if (!source.is_string()) {
          return error_ctx{error_code::syntax_error};
       }
-      result = source.get<std::string>();
+      result = source.template get<std::string>();
       return {};
    }
 
-   // Specialization for integer types (convert from double)
-   template <class T>
-      requires(int_t<T>)
-   error_ctx convert_from_generic(T& result, const generic& source)
+   // Specialization for integer types (convert from integer types or double)
+   template <class T, num_mode Mode>
+      requires(std::integral<T> && !std::same_as<T, bool> && !std::same_as<T, int64_t> && !std::same_as<T, uint64_t>)
+   error_ctx convert_from_generic(T& result, const generic_json<Mode>& source)
    {
       if (!source.is_number()) {
          return error_ctx{error_code::syntax_error};
       }
-      result = static_cast<T>(source.get<double>());
+      if constexpr (Mode == num_mode::u64) {
+         if (source.template holds<uint64_t>()) {
+            result = static_cast<T>(source.template get<uint64_t>());
+         }
+         else if (source.template holds<int64_t>()) {
+            result = static_cast<T>(source.template get<int64_t>());
+         }
+         else {
+            result = static_cast<T>(source.template get<double>());
+         }
+      }
+      else if constexpr (Mode == num_mode::i64) {
+         if (source.template holds<int64_t>()) {
+            result = static_cast<T>(source.template get<int64_t>());
+         }
+         else {
+            result = static_cast<T>(source.template get<double>());
+         }
+      }
+      else {
+         result = static_cast<T>(source.template get<double>());
+      }
       return {};
    }
 
    // Specialization for array-like containers
-   template <class T>
+   template <class T, num_mode Mode>
       requires(readable_array_t<T> && !std::same_as<T, std::string>)
-   error_ctx convert_from_generic(T& result, const generic& source)
+   error_ctx convert_from_generic(T& result, const generic_json<Mode>& source)
    {
       if (!source.is_array()) {
          return error_ctx{error_code::syntax_error};
@@ -730,9 +1034,9 @@ namespace glz
    }
 
    // Specialization for map-like containers
-   template <class T>
+   template <class T, num_mode Mode>
       requires readable_map_t<T>
-   error_ctx convert_from_generic(T& result, const generic& source)
+   error_ctx convert_from_generic(T& result, const generic_json<Mode>& source)
    {
       if (!source.is_object()) {
          return error_ctx{error_code::syntax_error};
@@ -765,11 +1069,16 @@ namespace glz
    // Concept to determine if a type needs deserialization from generic
    // These are container types that can't be directly referenced from glz::generic
    // because they're not the exact types stored in generic's variant
+   template <class V, class GenericType>
+   concept needs_container_deserialization_for =
+      (readable_array_t<V> || readable_map_t<V>) && !std::same_as<V, GenericType> &&
+      !std::same_as<V, typename GenericType::array_t> && !std::same_as<V, typename GenericType::object_t> &&
+      !std::same_as<V, std::string> && !std::same_as<V, double> && !std::same_as<V, bool> &&
+      !std::same_as<V, std::nullptr_t>;
+
+   // Legacy concept for backward compatibility
    template <class V>
-   concept needs_container_deserialization =
-      (readable_array_t<V> || readable_map_t<V>) && !std::same_as<V, generic> && !std::same_as<V, generic::array_t> &&
-      !std::same_as<V, generic::object_t> && !std::same_as<V, std::string> && !std::same_as<V, double> &&
-      !std::same_as<V, bool> && !std::same_as<V, std::nullptr_t>;
+   concept needs_container_deserialization = needs_container_deserialization_for<V, generic>;
 
    // Overload of get for glz::generic that deserializes containers
    // This overload is only selected for container types that need deserialization
