@@ -12,7 +12,9 @@
 #include "glaze/core/read.hpp"
 #include "glaze/core/reflect.hpp"
 #include "glaze/file/file_ops.hpp"
+#include "glaze/toml/common.hpp"
 #include "glaze/toml/opts.hpp"
+#include "glaze/toml/skip.hpp"
 #include "glaze/util/glaze_fast_float.hpp"
 #include "glaze/util/parse.hpp"
 #include "glaze/util/type_traits.hpp"
@@ -23,61 +25,16 @@ namespace glz
    struct parse<TOML>
    {
       template <auto Opts, class T, is_context Ctx, class It0, class It1>
-      GLZ_ALWAYS_INLINE static void op(T&& value, Ctx&& ctx, It0&& it, It1&& end)
+      GLZ_ALWAYS_INLINE static void op(T&& value, Ctx&& ctx, It0&& it, It1 end)
       {
-         from<TOML, std::decay_t<T>>::template op<Opts>(std::forward<T>(value), std::forward<Ctx>(ctx),
-                                                        std::forward<It0>(it), std::forward<It1>(end));
+         using V = std::remove_cvref_t<T>;
+         from<TOML, V>::template op<Opts>(std::forward<T>(value), std::forward<Ctx>(ctx), std::forward<It0>(it), end);
       }
    };
 
-   // Skip whitespace and comments
-   template <class It, class End>
-   GLZ_ALWAYS_INLINE void skip_ws_and_comments(It&& it, End&& end) noexcept
-   {
-      while (it != end) {
-         if (*it == ' ' || *it == '\t') {
-            ++it;
-         }
-         else if (*it == '#') {
-            // Skip comment to end of line
-            while (it != end && *it != '\n' && *it != '\r') {
-               ++it;
-            }
-         }
-         else {
-            break;
-         }
-      }
-   }
-
-   // Skip to next line
-   template <class Ctx, class It, class End>
-   GLZ_ALWAYS_INLINE bool skip_to_next_line(Ctx&, It&& it, End&& end) noexcept
-   {
-      while (it != end && *it != '\n' && *it != '\r') {
-         ++it;
-      }
-
-      if (it == end) {
-         return false;
-      }
-
-      if (*it == '\r') {
-         ++it;
-         if (it != end && *it == '\n') {
-            ++it;
-         }
-      }
-      else if (*it == '\n') {
-         ++it;
-      }
-
-      return true;
-   }
-
    // Parse TOML key (bare key or quoted key)
    template <class Ctx, class It, class End>
-   GLZ_ALWAYS_INLINE bool parse_toml_key(std::string& key, Ctx& ctx, It&& it, End&& end) noexcept
+   GLZ_ALWAYS_INLINE bool parse_toml_key(std::string& key, Ctx& ctx, It&& it, End end) noexcept
    {
       key.clear();
       skip_ws_and_comments(it, end);
@@ -245,7 +202,7 @@ namespace glz
    struct from<TOML, T>
    {
       template <auto Opts, is_context Ctx, class It0, class It1>
-      static void op(auto&& value, Ctx&& ctx, It0&& it, It1&& end)
+      static void op(auto&& value, Ctx&& ctx, It0&& it, It1 end)
       {
          using V = decltype(get_member(std::declval<T>(), meta_wrapper_v<T>));
          from<TOML, V>::template op<Opts>(get_member(value, meta_wrapper_v<T>), std::forward<Ctx>(ctx),
@@ -253,11 +210,286 @@ namespace glz
       }
    };
 
+   namespace detail
+   {
+      constexpr bool is_toml_base_specifier(char c) noexcept { return (c == 'x' || c == 'b' || c == 'o'); }
+
+      constexpr int toml_specified_base(char c) noexcept
+      {
+         switch (c) {
+         case 'x':
+            return 16;
+         case 'b':
+            return 2;
+         case 'o':
+            return 8;
+         }
+
+         // Should never reach here, but seems a good default.
+         return 10;
+      }
+
+      constexpr bool is_any_toml_digit(char c) noexcept
+      {
+         if (c >= '0' && c <= '9') {
+            return true;
+         }
+
+         if (c >= 'a' && c <= 'f') {
+            return true;
+         }
+
+         if (c >= 'A' && c <= 'F') {
+            return true;
+         }
+
+         return false;
+      }
+
+      constexpr bool is_valid_toml_digit(char c, int base) noexcept
+      {
+         if (base <= 10) {
+            return c >= '0' && c < ('0' + base);
+         }
+
+         return is_any_toml_digit(c);
+      }
+
+      struct unsigned_accumulator
+      {
+        private:
+         int base = 10;
+
+        public:
+         constexpr void inform_base(int base) noexcept { this->base = base; }
+
+         constexpr bool is_valid_digit(char c) const noexcept { return is_valid_toml_digit(c, this->base); }
+
+         template <std::unsigned_integral T>
+         constexpr bool try_accumulate(T& v, int digit) const noexcept
+         {
+            if (digit < 0 || digit >= this->base) [[unlikely]] {
+               return false;
+            }
+
+            // Check overflow.
+            const auto max_without_overflow = ((std::numeric_limits<T>::max)() - digit) / this->base;
+            if (v > max_without_overflow) [[unlikely]] {
+               return false;
+            }
+
+            v = v * this->base + digit;
+
+            return true;
+         }
+      };
+
+      struct signed_accumulator
+      {
+        private:
+         int base = 10;
+         bool negated = false;
+
+        public:
+         constexpr void inform_negated() noexcept { this->negated = true; }
+
+         constexpr void inform_base(int base) noexcept { this->base = base; }
+
+         constexpr bool is_valid_digit(char c) const noexcept { return is_valid_toml_digit(c, this->base); }
+
+         template <std::signed_integral T>
+         constexpr bool try_accumulate(T& v, int digit) const noexcept
+         {
+            if (digit < 0 || digit >= this->base) [[unlikely]] {
+               return false;
+            }
+
+            if (this->negated) {
+               // Check underflow.
+               const auto min_without_underflow = ((std::numeric_limits<T>::min)() + digit) / this->base;
+               if (v < min_without_underflow) [[unlikely]] {
+                  return false;
+               }
+
+               v = v * this->base - digit;
+
+               return true;
+            }
+
+            // Check overflow.
+            const auto max_without_overflow = ((std::numeric_limits<T>::max)() - digit) / this->base;
+            if (v > max_without_overflow) [[unlikely]] {
+               return false;
+            }
+
+            v = v * this->base + digit;
+
+            return true;
+         }
+      };
+
+      template <std::integral T>
+      constexpr bool parse_toml_integer(T& v, auto&& it, auto end) noexcept
+      {
+         // Ensure the caller gave us at least one character.
+         if (it == end) [[unlikely]] {
+            return false;
+         }
+
+         auto accumulator = [] {
+            if constexpr (std::unsigned_integral<T>) {
+               return unsigned_accumulator{};
+            }
+            else {
+               return signed_accumulator{};
+            }
+         }();
+
+         bool saw_negative_sign = false;
+         bool saw_sign = false; // Tracks whether +/- was supplied so prefixed bases can reject it.
+
+         if (*it == '-') {
+            saw_negative_sign = true;
+            saw_sign = true;
+            ++it;
+
+            if (it == end) [[unlikely]] {
+               return false;
+            }
+
+            if constexpr (std::signed_integral<T>) {
+               accumulator.inform_negated();
+            }
+         }
+
+         if (*it == '+') {
+            // Only a single leading sign is permitted.
+            if (saw_sign) [[unlikely]] {
+               return false;
+            }
+
+            ++it;
+            saw_sign = true;
+
+            if (it == end) [[unlikely]] {
+               return false;
+            }
+         }
+
+         if (it == end) [[unlikely]] {
+            return false;
+         }
+
+         if (*it == '_') [[unlikely]] {
+            return false;
+         }
+
+         int base = 10;
+         bool base_overridden = false;
+
+         if (*it == '0') {
+            auto next = it;
+            ++next;
+
+            if (next == end) {
+               it = next;
+               v = 0;
+               return true;
+            }
+
+            if (*next == '_') [[unlikely]] {
+               return false;
+            }
+
+            if (is_toml_base_specifier(*next)) {
+               // Switching to hexadecimal/octal/binary after 0.
+               base = toml_specified_base(*next);
+               base_overridden = true;
+
+               it = next;
+               ++it;
+
+               if (it == end) [[unlikely]] {
+                  return false;
+               }
+
+               if (not is_valid_toml_digit(*it, base)) [[unlikely]] {
+                  return false;
+               }
+            }
+            else if (is_any_toml_digit(*next)) [[unlikely]] {
+               // Decimal literals like 0123 are invalid.
+               return false;
+            }
+            else {
+               it = next;
+               v = 0;
+               return true;
+            }
+         }
+
+         if (base_overridden) {
+            accumulator.inform_base(base);
+
+            // Only non-negative values are allowed for prefixed bases.
+            if (saw_sign) [[unlikely]] {
+               return false;
+            }
+         }
+
+         v = 0;
+         bool saw_digit = false;
+
+         // Consume digits (with optional underscores) while respecting overflow limits.
+         while (true) {
+            if (it == end) {
+               break;
+            }
+
+            if (*it == '_') {
+               ++it;
+
+               if (it == end) [[unlikely]] {
+                  return false;
+               }
+
+               if (not is_any_toml_digit(*it)) [[unlikely]] {
+                  return false;
+               }
+            }
+            else if (not is_any_toml_digit(*it)) {
+               break;
+            }
+
+            const auto digit = char_to_digit(*it);
+
+            if (not accumulator.try_accumulate(v, digit)) {
+               return false;
+            }
+
+            saw_digit = true;
+            ++it;
+         }
+
+         if (not saw_digit) [[unlikely]] {
+            return false;
+         }
+
+         if constexpr (std::unsigned_integral<T>) {
+            if (saw_negative_sign && v != 0) [[unlikely]] {
+               return false;
+            }
+         }
+
+         return true;
+      }
+   }
+
    template <num_t T>
    struct from<TOML, T>
    {
       template <auto Opts, class It>
-      static void op(auto&& value, is_context auto&& ctx, It&& it, auto&& end) noexcept
+      static void op(auto&& value, is_context auto&& ctx, It&& it, auto end) noexcept
       {
          if (bool(ctx.error)) [[unlikely]] {
             return;
@@ -272,45 +504,9 @@ namespace glz
 
          using V = decay_keep_volatile_t<decltype(value)>;
          if constexpr (int_t<V>) {
-            if constexpr (std::is_unsigned_v<V>) {
-               uint64_t i{};
-               if (*it == '-') [[unlikely]] {
-                  ctx.error = error_code::parse_number_failure;
-                  return;
-               }
-
-               if (not glz::atoi(i, it, end)) [[unlikely]] {
-                  ctx.error = error_code::parse_number_failure;
-                  return;
-               }
-
-               if (i > (std::numeric_limits<V>::max)()) [[unlikely]] {
-                  ctx.error = error_code::parse_number_failure;
-                  return;
-               }
-               value = static_cast<V>(i);
-            }
-            else {
-               uint64_t i{};
-               int sign = 1;
-               if (*it == '-') {
-                  sign = -1;
-                  ++it;
-                  if (it == end) [[unlikely]] {
-                     ctx.error = error_code::unexpected_end;
-                     return;
-                  }
-               }
-               if (not glz::atoi(i, it, end)) [[unlikely]] {
-                  ctx.error = error_code::parse_number_failure;
-                  return;
-               }
-
-               if (i > (std::numeric_limits<V>::max)()) [[unlikely]] {
-                  ctx.error = error_code::parse_number_failure;
-                  return;
-               }
-               value = sign * static_cast<V>(i);
+            if (not detail::parse_toml_integer(value, it, end)) [[unlikely]] {
+               ctx.error = error_code::parse_number_failure;
+               return;
             }
          }
          else {
@@ -328,7 +524,7 @@ namespace glz
    struct from<TOML, T>
    {
       template <auto Opts, class It>
-      static void op(auto&& value, is_context auto&& ctx, It&& it, auto&& end)
+      static void op(auto&& value, is_context auto&& ctx, It&& it, auto end)
       {
          if (bool(ctx.error)) [[unlikely]] {
             return;
@@ -539,7 +735,7 @@ namespace glz
    struct from<TOML, T>
    {
       template <auto Opts, class It>
-      static void op(auto&& value, is_context auto&& ctx, It&& it, auto&& end) noexcept
+      static void op(auto&& value, is_context auto&& ctx, It&& it, auto end) noexcept
       {
          if (bool(ctx.error)) [[unlikely]] {
             return;
@@ -573,7 +769,7 @@ namespace glz
    struct from<TOML, T>
    {
       template <auto Opts, class It>
-      static void op(auto&& value, is_context auto&& ctx, It&& it, auto&& end)
+      static void op(auto&& value, is_context auto&& ctx, It&& it, auto end)
       {
          if (bool(ctx.error)) [[unlikely]] {
             return;
@@ -597,7 +793,7 @@ namespace glz
 
          size_t index = 0;
          while (it != end) {
-            using value_type = typename std::decay_t<T>::value_type;
+            using value_type = typename std::remove_cvref_t<T>::value_type;
 
             if constexpr (emplace_backable<T>) {
                auto& element = value.emplace_back();
@@ -643,10 +839,11 @@ namespace glz
    namespace detail
    {
       template <auto Opts, class T, class It, class End, class Ctx>
-      GLZ_ALWAYS_INLINE void parse_toml_object_members(T&& value, It&& it, End&& end, Ctx&& ctx, bool is_inline_table)
+      GLZ_ALWAYS_INLINE void parse_toml_object_members(T&& value, It&& it, End end, Ctx&& ctx, bool is_inline_table)
       {
-         static constexpr auto N = reflect<std::decay_t<T>>::size;
-         static constexpr auto HashInfo = hash_info<std::decay_t<T>>;
+         using U = std::remove_cvref_t<T>;
+         static constexpr auto N = reflect<U>::size;
+         static constexpr auto HashInfo = hash_info<U>;
          // TODO: Think if we should make inline table a constexpr to reduce runtime checks
          // TODO: Think if it's feasible to write a function to find out the deepest nested structure
          // and use it instead of vector
@@ -703,22 +900,24 @@ namespace glz
                return;
             }
 
-            const auto index = decode_hash_with_size<TOML, std::decay_t<T>, HashInfo, HashInfo.type>::op(
+            const auto index = decode_hash_with_size<TOML, U, HashInfo, HashInfo.type>::op(
                key_str.data(), key_str.data() + key_str.size(), key_str.size());
 
-            if (index < N) [[likely]] {
+            const bool key_matches = index < N && key_str == reflect<U>::keys[index];
+
+            if (key_matches) [[likely]] {
                visit<N>(
                   [&]<size_t I>() {
                      if (I == index) {
                         decltype(auto) member_obj = [&]() -> decltype(auto) {
-                           if constexpr (reflectable<std::decay_t<T>>) {
+                           if constexpr (reflectable<U>) {
                               // For reflectable types, to_tie provides access to members
                               return get<I>(to_tie(value));
                            }
                            else {
                               // For glaze_object_t, reflect<T>::values gives member metadata
                               // and get_member accesses the actual member value
-                              return get_member(value, get<I>(reflect<std::decay_t<T>>::values));
+                              return get_member(value, get<I>(reflect<U>::values));
                            }
                         }();
 
@@ -726,6 +925,7 @@ namespace glz
                         using member_type = std::decay_t<decltype(member_obj)>;
                         from<TOML, member_type>::template op<Opts>(member_obj, ctx, it, end);
                      }
+                     return !bool(ctx.error);
                   },
                   index);
 
@@ -734,16 +934,14 @@ namespace glz
                }
             }
             else {
-               // Unknown key
-               // TODO: Add option to error on unknown keys or ignore them
-               // For now, skip the value associated with the unknown key.
-               // This requires parsing the value to know where it ends.
-               // A simpler skip to next line might not work for inline tables or complex values.
-               // For now, let's try a simple skip to comma or end of line/table.
-               while (it != end) {
-                  if (is_inline_table && (*it == ',' || *it == '}')) break;
-                  if (!is_inline_table && (*it == '\n' || *it == '\r')) break;
-                  ++it;
+               if constexpr (Opts.error_on_unknown_keys) {
+                  ctx.error = error_code::unknown_key;
+                  return;
+               }
+
+               skip_value<TOML>::template op<Opts>(ctx, it, end);
+               if (bool(ctx.error)) [[unlikely]] {
+                  return;
                }
             }
 
@@ -791,20 +989,24 @@ namespace glz
          return true;
       }
       else {
-         static constexpr auto N = reflect<std::decay_t<T>>::size;
-         static constexpr auto HashInfo = hash_info<std::decay_t<T>>;
-         const auto index = decode_hash_with_size<TOML, std::decay_t<T>, HashInfo, HashInfo.type>::op(
+         using U = std::remove_cvref_t<T>;
+         static constexpr auto N = reflect<U>::size;
+         static constexpr auto HashInfo = hash_info<U>;
+         const auto index = decode_hash_with_size<TOML, U, HashInfo, HashInfo.type>::op(
             path.front().data(), path.front().data() + path.front().size(), path.front().size());
-         if (index < N) [[likely]] {
+
+         const bool key_matches = index < N && path.front() == reflect<U>::keys[index];
+
+         if (key_matches) [[likely]] {
             visit<N>(
                [&]<size_t I>() {
                   if (I == index) {
                      decltype(auto) member_obj = [&]() -> decltype(auto) {
-                        if constexpr (reflectable<std::decay_t<T>>) {
+                        if constexpr (reflectable<U>) {
                            return get<I>(to_tie(root));
                         }
                         else {
-                           return get_member(root, get<I>(reflect<std::decay_t<T>>::values));
+                           return get_member(root, get<I>(reflect<U>::values));
                         }
                      }();
 
@@ -820,6 +1022,15 @@ namespace glz
                },
                index);
          }
+         else {
+            if constexpr (Opts.error_on_unknown_keys) {
+               ctx.error = error_code::unknown_key;
+               return false;
+            }
+
+            skip_value<TOML>::template op<Opts>(ctx, it, end);
+            return !bool(ctx.error);
+         }
          return !bool(ctx.error);
       }
    }
@@ -829,7 +1040,7 @@ namespace glz
    struct from<TOML, T>
    {
       template <auto Opts, class It>
-      static void op(auto&& value, is_context auto&& ctx, It&& it, auto&& end)
+      static void op(auto&& value, is_context auto&& ctx, It&& it, auto end)
       {
          while (it != end) {
             // TODO: Introduce OPTS here
